@@ -1,7 +1,7 @@
 package AppBase::Grep;
 
-our $DATE = '2018-01-26'; # DATE
-our $VERSION = '0.001'; # VERSION
+our $DATE = '2018-01-29'; # DATE
+our $VERSION = '0.002'; # VERSION
 
 use 5.010001;
 use strict;
@@ -9,6 +9,13 @@ use warnings;
 
 our %SPEC;
 
+our %Colors = (
+    label     => "\e[35m",   # magenta
+    separator => "\e[36m",   # cyan
+    linum     => "\e[32m",   # green
+    match     => "\e[1;31m", # bold red
+);
+
 $SPEC{grep} = {
     v => 1.1,
     summary => 'A base for grep-like CLI utilities',
@@ -45,7 +52,6 @@ _
         },
         color => {
             schema => ['str*', in=>[qw/never always auto/]],
-            default => 'auto',
             tags => ['category:general-output-control'],
         },
         quiet => {
@@ -53,6 +59,12 @@ _
             cmdline_aliases => {silent=>{}, q=>{}},
             tags => ['category:general-output-control'],
         },
+
+        line_number => {
+            schema => ['true*'],
+            cmdline_aliases => {n=>{}},
+            tags => ['category:output-line-prefix-control'],
+        },
         # XXX max_count
         # word_regexp (-w) ?
         # line_regexp (-x) ?
@@ -68,9 +80,13 @@ sub grep {
     my $opt_invert = $args{invert_match};
     my $opt_count  = $args{count};
     my $opt_quiet  = $args{quiet};
+    my $opt_linum  = $args{line_number};
     my $pat        = $opt_ci ? qr/$args{pattern}/i : qr/$args{pattern}/;
 
-    my $color = $args{color} // 'auto';
+    my $color = $args{color} //
+        (defined $ENV{COLOR} ? ($ENV{COLOR} ? 'always' : 'never') : undef) //
+        'auto';
+
     my $use_color;
     if ($color eq 'always') {
         $use_color = 1;
@@ -81,24 +97,54 @@ sub grep {
     }
     my $source = $args{_source};
 
-    my $linum = 0;
     my $num_matches = 0;
-    my ($line, $label);
+    my ($line, $label, $linum);
 
     my $code_print = sub {
-        my $highlight = shift;
-        if ($highlight) {
-            $line =~ s/($pat)/\e[1;31m$1\e[0m/g;
+        if (defined $label && length $label) {
+            if ($use_color) {
+                print "$Colors{label}$label\e[0m$Colors{separator}:\e[0m";
+            } else {
+                print $label, ":";
+            }
+        }
+
+        if ($opt_linum) {
+            if ($use_color) {
+                print "$Colors{linum}$linum\e[0m$Colors{separator}:\e[0m";
+            } else {
+                print $linum, ":";
+            }
+        }
+
+        if ($use_color && !$opt_invert) {
+            $line =~ s/($pat)/$Colors{match}$1\e[0m/g;
             print $line;
         } else {
             print $line;
         }
     };
 
+    my $prevlabel;
     while (1) {
         ($line, $label) = $source->();
         last unless defined $line;
-        $linum++;
+
+        $label //= '';
+
+        if ($opt_linum) {
+            if (!defined $prevlabel) {
+                $prevlabel = $label;
+                $linum = 1;
+            } else {
+                if ($label ne $prevlabel) {
+                    $prevlabel = $label;
+                    $linum = 1;
+                } else {
+                    $linum++;
+                }
+            }
+        }
 
         my $is_match = 0;
         if ($line =~ $pat) {
@@ -107,14 +153,14 @@ sub grep {
             if ($opt_quiet || $opt_count) {
                 $num_matches++;
             } else {
-                $code_print->($use_color);
+                $code_print->();
             }
         } else {
             next unless $opt_invert;
             if ($opt_quiet || $opt_count) {
                 $num_matches++;
             } else {
-                $code_print->(0);
+                $code_print->();
             }
         }
     }
@@ -142,7 +188,7 @@ AppBase::Grep - A base for grep-like CLI utilities
 
 =head1 VERSION
 
-This document describes version 0.001 of AppBase::Grep (from Perl distribution AppBase-Grep), released on 2018-01-26.
+This document describes version 0.002 of AppBase::Grep (from Perl distribution AppBase-Grep), released on 2018-01-29.
 
 =head1 FUNCTIONS
 
@@ -165,7 +211,7 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<color> => I<str> (default: "auto")
+=item * B<color> => I<str>
 
 =item * B<count> => I<true>
 
@@ -177,6 +223,8 @@ Supress normal output, return a count of matching lines.
 
 Invert the sense of matching.
 
+=item * B<line_number> => I<true>
+
 =item * B<pattern>* => I<re>
 
 =item * B<quiet> => I<true>
@@ -194,6 +242,14 @@ that contains extra information.
 
 Return value:  (any)
 
+=head1 ENVIRONMENT
+
+=head2 COLOR
+
+Boolean. If set to true, will set default C<--color> to C<always> instead of
+C<auto>. If set to false, will set default C<--color> to C<never> instead of
+C<auto>. This behavior is not in GNU grep.
+
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/AppBase-Grep>.
